<style>
.default { }
.error { color: red; }
.list { color: blue; }
.alt { font-style: italic; }
.titled { text-decoration: underline dotted; }
.right { text-align: right; }
.center { text-align: center; }

caption { font-style: italic; margin-bottom: 1em; }

table {
  border: 1px solid #eaeaea;
  border-spacing: 0;
  border-collapse: collapse;
  table-layout: auto;
  margin: 1em auto;
}

tr { vertical-align: middle; }

th {
  border: 1px solid #eaeaea;
  padding: 0.5rem;
  background: var(--color-bg-box);
}

td {
  vertical-align: top;
  padding: 0.5rem;
  border: 1px solid #eaeaea;
}
</style>
