<!DOCTYPE html>
<html lang=en>
<head>
<meta charset="utf-8">
<meta name="viewport" content="width=device-width, initial-scale=1.0">
{% if table.title %}<title>{{ table.title }}</title>{% endif %}
{% include "style.html" %}
</head>
<body>
{% include "html_bare.html" %}
</body>
</html>
